@@ -36,6 +36,11 @@
 #include <time.h>
 #include <math.h>
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+#include <share.h>
+#endif
+
 gboolean gconf_log_debug_messages = FALSE;
 
 static gboolean gconf_daemon_mode = FALSE;
@@ -129,14 +134,14 @@ gconf_file_exists (const gchar* filename)
   
   g_return_val_if_fail (filename != NULL,FALSE);
   
-  return stat (filename, &s) == 0;
+  return g_stat (filename, &s) == 0;
 }
 
 gboolean
 gconf_file_test(const gchar* filename, int test)
 {
   struct stat s;
-  if(stat (filename, &s) != 0)
+  if(g_stat (filename, &s) != 0)
     return FALSE;
   if(!(test & GCONF_FILE_ISFILE) && S_ISREG(s.st_mode))
     return FALSE;
@@ -715,6 +720,34 @@ unquote_string(gchar* s)
   return s;
 }
 
+#ifdef G_OS_WIN32
+
+/* Return the home directory with forward slashes instead of backslashes */
+
+const char*
+_gconf_win32_get_home_dir (void)
+{
+  static GQuark quark = 0;
+  char *home_copy, *p;
+
+  if (quark != 0)
+    return g_quark_to_string (quark);
+  
+  home_copy = g_strdup (g_get_home_dir ());
+
+  /* Replace backslashes with forward slashes */
+  for (p = home_copy; *p; p++)
+    if (*p == '\\')
+      *p = '/';
+  
+  quark = g_quark_from_string (home_copy);
+  g_free (home_copy);
+
+  return g_quark_to_string (quark);
+}
+
+#endif
+
 static const gchar*
 get_variable(const gchar* varname)
 {
@@ -723,7 +756,11 @@ get_variable(const gchar* varname)
   */
   if (strcmp(varname, "HOME") == 0)
     {
-      return g_get_home_dir();
+#ifndef G_OS_WIN32
+      return g_get_home_dir ();
+#else
+      return _gconf_win32_get_home_dir ();
+#endif
     }
   else if (strcmp(varname, "USER") == 0)
     {
@@ -736,7 +773,7 @@ get_variable(const gchar* varname)
     {
       /* This is magic: if a variable called ENV_FOO is used,
          then the environment variable FOO is checked */
-      gchar* envvar = getenv(&varname[4]);
+      const gchar* envvar = g_getenv(&varname[4]);
 
       if (envvar)
         return envvar;
@@ -828,7 +865,7 @@ gconf_load_source_path(const gchar* filename, GError** err)
   GSList *l = NULL;
   gchar buf[512];
 
-  f = fopen(filename, "r");
+  f = g_fopen(filename, "r");
 
   if (f == NULL)
     {
@@ -836,7 +873,7 @@ gconf_load_source_path(const gchar* filename, GError** err)
         *err = gconf_error_new(GCONF_ERROR_FAILED,
                                _("Couldn't open path file `%s': %s\n"), 
                                filename, 
-                               strerror(errno));
+                               g_strerror(errno));
       return NULL;
     }
 
@@ -867,11 +904,18 @@ gconf_load_source_path(const gchar* filename, GError** err)
           unq = unquote_string(s);
 
           varsubst = subst_variables (unq);
+#ifdef G_OS_WIN32
+	  {
+	    gchar *tem = varsubst;
+	    varsubst = _gconf_win32_replace_prefix (varsubst);
+	    g_free (tem);
+	  }
+#endif
           included = gconf_load_source_path (varsubst, NULL);
           g_free (varsubst);
           
           if (included != NULL)
-            g_slist_concat (l, included);
+            l = g_slist_concat (l, included);
         }
       else 
         {
@@ -900,7 +944,7 @@ gconf_load_source_path(const gchar* filename, GError** err)
         *err = gconf_error_new(GCONF_ERROR_FAILED,
                                _("Read error on file `%s': %s\n"), 
                                filename,
-                               strerror(errno));
+                               g_strerror(errno));
       /* don't return, we want to go ahead and return any 
          addresses we already loaded. */
     }
@@ -1086,15 +1130,19 @@ gconf_current_locale(void)
  * Log
  */
 
+#ifdef HAVE_SYSLOG_H
 #include <syslog.h>
+#endif
 
 void
 gconf_log(GConfLogPriority pri, const gchar* fmt, ...)
 {
   gchar* msg;
-  gchar* convmsg;
   va_list args;
+#ifdef HAVE_SYSLOG_H
+  gchar* convmsg;
   int syslog_pri = LOG_DEBUG;
+#endif
 
   if (!gconf_log_debug_messages && 
       pri == GCL_DEBUG)
@@ -1104,6 +1152,7 @@ gconf_log(GConfLogPriority pri, const gchar* fmt, ...)
   msg = g_strdup_vprintf(fmt, args);
   va_end (args);
 
+#ifdef HAVE_SYSLOG_H
   if (gconf_daemon_mode)
     {
       switch (pri)
@@ -1155,6 +1204,7 @@ gconf_log(GConfLogPriority pri, const gchar* fmt, ...)
 	}
     }
   else
+#endif
     {
       switch (pri)
         {
@@ -2225,9 +2275,12 @@ gconf_lock_destroy (GConfLock* lock)
   g_free (lock);
 }
 
+#ifndef G_OS_WIN32
+
 static void
 set_close_on_exec (int fd)
 {
+#if defined (F_GETFD) && defined (FD_CLOEXEC)
   int val;
 
   val = fcntl (fd, F_GETFD, 0);
@@ -2241,8 +2294,12 @@ set_close_on_exec (int fd)
 
   if (fcntl (fd, F_SETFD, val) < 0)
     gconf_log (GCL_DEBUG, "couldn't F_SETFD: %s\n", g_strerror (errno));
+#endif
 }
 
+#endif
+
+#ifdef F_SETLK
 /* Your basic Stevens cut-and-paste */
 static int
 lock_reg (int fd, int cmd, int type, off_t offset, int whence, off_t len)
@@ -2256,15 +2313,25 @@ lock_reg (int fd, int cmd, int type, off_t offset, int whence, off_t len)
 
   return fcntl (fd, cmd, &lock);
 }
+#endif
 
+#ifdef F_SETLK
 #define lock_entire_file(fd) \
   lock_reg ((fd), F_SETLK, F_WRLCK, 0, SEEK_SET, 0)
 #define unlock_entire_file(fd) \
   lock_reg ((fd), F_SETLK, F_UNLCK, 0, SEEK_SET, 0)
+#elif defined (G_OS_WIN32)
+/* We don't use these macros */
+#else
+#warning Please implement proper locking
+#define lock_entire_file(fd) 0
+#define unlock_entire_file(fd) 0
+#endif
 
 static gboolean
 file_locked_by_someone_else (int fd)
 {
+#ifdef F_SETLK
   struct flock lock;
 
   lock.l_type = F_WRLCK;
@@ -2279,8 +2346,13 @@ file_locked_by_someone_else (int fd)
     return FALSE; /* we have the lock */
   else
     return TRUE; /* someone else has it */
+#else
+  return FALSE;
+#endif
 }
 
+#ifndef G_OS_WIN32
+
 static char*
 unique_filename (const char *directory)
 {
@@ -2294,18 +2366,19 @@ unique_filename (const char *directory)
   return uniquefile;
 }
 
+#endif
+
 static int
 create_new_locked_file (const gchar *directory,
                         const gchar *filename,
                         GError     **err)
 {
   int fd;
-  char *uniquefile;
-  gboolean got_lock;
-  
-  got_lock = FALSE;
-  
-  uniquefile = unique_filename (directory);
+  gboolean got_lock = FALSE;
+
+#ifndef G_OS_WIN32
+
+  char *uniquefile = unique_filename (directory);
 
   fd = open (uniquefile, O_WRONLY | O_CREAT, 0700);
 
@@ -2357,6 +2430,25 @@ create_new_locked_file (const gchar *directory,
   unlink (uniquefile);
   g_free (uniquefile);
 
+#else
+
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+      fd = _wsopen (wfilename, O_WRONLY|O_CREAT|O_EXCL, SH_DENYWR, 0700);
+      g_free (wfilename);
+    }
+  else
+    {
+      char *cpfilename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+      fd = _sopen (cpfilename, O_WRONLY|O_CREAT|O_EXCL, SH_DENYWR, 0700);
+      g_free (cpfilename);
+    }
+
+  got_lock = (fd >= 0);
+
+#endif
+
   if (!got_lock)
     {
       if (fd >= 0)
@@ -2383,6 +2475,9 @@ open_empty_locked_file (const gchar *directory,
    * existed; try to get the lock on the existing file, and if we can
    * get that lock, delete it, then start over.
    */
+
+#ifndef G_OS_WIN32
+
   fd = open (filename, O_RDWR, 0700);
   if (fd < 0)
     {
@@ -2401,7 +2496,7 @@ open_empty_locked_file (const gchar *directory,
                    GCONF_ERROR,
                    GCONF_ERROR_LOCK_FAILED,
                    _("Failed to lock '%s': probably another process has the lock, or your operating system has NFS file locking misconfigured (%s)"),
-                   filename, strerror (errno));
+                   filename, g_strerror (errno));
       close (fd);
       return -1;
     }
@@ -2412,6 +2507,20 @@ open_empty_locked_file (const gchar *directory,
   close (fd);
   fd = -1;
 
+#else
+  /* Unlinking open files fail on Win32 */
+
+  if (g_remove (filename) == -1)
+    {
+      g_set_error (err,
+		   GCONF_ERROR,
+		   GCONF_ERROR_LOCK_FAILED,
+		   _("Failed to remove '%s': %s"),
+		   filename, g_strerror (errno));
+      return -1;
+    }
+#endif
+
   /* Now retry creating our file */
   fd = create_new_locked_file (directory, filename, err);
   
@@ -2424,7 +2533,7 @@ read_current_server_and_set_warning (const gchar *iorfile,
 {
   FILE *fp;
   
-  fp = fopen (iorfile, "r");
+  fp = g_fopen (iorfile, "r");
           
   if (fp == NULL)
     {
@@ -2532,7 +2641,7 @@ gconf_get_lock_or_current_holder (const gchar  *lock_directory,
   if (current_server)
     *current_server = CORBA_OBJECT_NIL;
   
-  if (mkdir (lock_directory, 0700) < 0 &&
+  if (g_mkdir (lock_directory, 0700) < 0 &&
       errno != EEXIST)
     {
       gconf_set_error (err,
@@ -2599,7 +2708,7 @@ gconf_get_lock_or_current_holder (const gchar  *lock_directory,
                            _("Can't write to file `%s': %s"),
                            lock->iorfile, g_strerror (errno));
 
-          unlink (lock->iorfile);
+          g_unlink (lock->iorfile);
           gconf_lock_destroy (lock);
 
           return NULL;
@@ -2642,6 +2751,8 @@ gconf_release_lock (GConfLock *lock,
       goto out;
     }
 
+#ifndef G_OS_WIN32
+
   /* To avoid annoying .nfs3435314513453145 files on unlink, which keep us
    * from removing the lock directory, we don't want to hold the
    * lockfile open after removing all links to it. But we can't
@@ -2676,6 +2787,8 @@ gconf_release_lock (GConfLock *lock,
       goto out;
     }
 
+#endif
+
   /* Now drop our lock */
   if (lock->lock_fd >= 0)
     {
@@ -2683,6 +2796,8 @@ gconf_release_lock (GConfLock *lock,
       lock->lock_fd = -1;
     }
 
+#ifndef G_OS_WIN32
+
   /* Now remove the temporary link we used to avoid .nfs351453 garbage */
   if (unlink (uniquefile) < 0)
     {
@@ -2695,10 +2810,12 @@ gconf_release_lock (GConfLock *lock,
       goto out;
     }
 
+#endif
+
   /* And finally clean up the directory - this would have failed if
    * we had .nfs323423423 junk
    */
-  if (rmdir (lock->lock_directory) < 0)
+  if (g_rmdir (lock->lock_directory) < 0)
     {
       g_set_error (err,
                    GCONF_ERROR,
@@ -2744,7 +2861,7 @@ gconf_daemon_blow_away_locks (void)
 
   iorfile = g_strconcat (lock_directory, "/ior", NULL);
 
-  if (unlink (iorfile) < 0)
+  if (g_unlink (iorfile) < 0)
     g_printerr (_("Failed to unlink lock file %s: %s\n"),
                 iorfile, g_strerror (errno));
 
@@ -2820,7 +2937,14 @@ gconf_get_daemon_dir (void)
       return s;
     }
   else
-    return g_strconcat (g_get_home_dir (), "/.gconfd", NULL);
+    {
+#ifndef G_OS_WIN32
+      const char *home = g_get_home_dir ();
+#else
+      const char *home = _gconf_win32_get_home_dir ();
+#endif
+      return g_strconcat (home, "/.gconfd", NULL);
+    }
 }
 
 char*
@@ -2836,6 +2960,8 @@ gconf_get_lock_dir (void)
   return lock_dir;
 }
 
+#if defined (F_SETFD) && defined (FD_CLOEXEC)
+
 static void
 set_cloexec (gint fd)
 {
@@ -2859,6 +2985,12 @@ close_fd_func (gpointer data)
     }
 }
 
+#else
+
+#define close_fd_func NULL
+
+#endif
+
 ConfigServer
 gconf_activate_server (gboolean  start_if_not_found,
                        GError  **error)
@@ -2879,7 +3011,7 @@ gconf_activate_server (gboolean  start_if_not_found,
   
   gconfd_dir = gconf_get_daemon_dir ();
   
-  dir_accessible = stat (gconfd_dir, &statbuf) >= 0;
+  dir_accessible = g_stat (gconfd_dir, &statbuf) >= 0;
 
   if (!dir_accessible && errno != ENOENT)
     {
@@ -2936,7 +3068,7 @@ gconf_activate_server (gboolean  start_if_not_found,
           goto out;
         }
 
-      argv[0] = g_strconcat (GCONF_SERVERDIR, "/" GCONFD, NULL);
+      argv[0] = g_build_filename (GCONF_SERVERDIR, GCONFD, NULL);
       argv[1] = g_strdup_printf ("%d", p[1]);
       argv[2] = NULL;
   
@@ -2946,7 +3078,7 @@ gconf_activate_server (gboolean  start_if_not_found,
                           NULL,
                           G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
                           close_fd_func,
-                          p,
+			  p,
                           NULL,
                           &tmp_err))
         {
